@@ -21,4 +21,9 @@ public interface Drawer {
      * @param drawingTools the givens drawing tools.
      */
     void draw(DrawingTools drawingTools);
+
+    /**
+     * @return true if it is a 2D drawing
+     */
+    boolean is2DView();
 }
