@@ -46,9 +46,9 @@ public final class G2DCanvas implements Canvas {
 
     private static final PickingManager PICKINGMANAGER = new PickingManager() {
 
-        @Override
-        public void addPickingTask(PickingTask pickingTask) { }
-    };
+            @Override
+            public void addPickingTask(PickingTask pickingTask) { }
+        };
 
     /**
      * The current mainDrawer.
@@ -74,6 +74,12 @@ public final class G2DCanvas implements Canvas {
         return drawingTools;
     }
 
+    public void setGraphics(Graphics2D g2d, int width, int height) {
+        this.dimension.width = width;
+        this.dimension.height = height;
+        this.motor.setGraphics(g2d);
+    }
+
     // Implementation of getter & setter from Canvas.
 
     @Override
