@@ -15,6 +15,7 @@ import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.GradientPaint;
 import java.awt.Graphics2D;
+import java.awt.LinearGradientPaint;
 import java.awt.Paint;
 import java.awt.RenderingHints;
 import java.awt.Shape;
@@ -23,16 +24,22 @@ import java.awt.geom.AffineTransform;
 import java.awt.geom.Area;
 import java.awt.geom.NoninvertibleTransformException;
 import java.awt.geom.Path2D;
+import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.DataBufferInt;
+import java.util.Arrays;
 
+import java.awt.geom.Point2D;
+import java.awt.MultipleGradientPaint;
 
 /**
  * @author Calixte DENIZET
  */
 public final class DrawTools {
 
-    private static final Stroke stroke = new BasicStroke(1f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
+    private static final Stroke stroke = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
+    private static final Stroke EMPTYSTROKE = new BasicStroke(0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
+    private static final Color TRANSLUCENT_BLACK = new Color(0, 0, 0, 0);
 
     /**
      * Fill a triangle in using a Gouraud shading
@@ -50,26 +57,20 @@ public final class DrawTools {
         double[] pv2 = get2DProjection(v2[0], v2[1], v0[0], v0[1], v1[0], v1[1]);
 
         Paint oldPaint = g2d.getPaint();
-
-        // When the colours are totally opaque, the combination of the two GradientPaint are equivalent to a Gouraud shading
-        // If there is transparency, that could lead to a different rendering between JOGL and this implementation.
-        GradientPaint gp = new GradientPaint((float) v0[0], (float) v0[1], t.colors[0], (float) pv0[0], (float) pv0[1], t.colors[1]);
-        g2d.setPaint(gp);
         Area area = new Area(contour);
         area.add(new Area(stroke.createStrokedShape(contour)));
-        g2d.fill(contour);
 
-        gp = new GradientPaint((float) v1[0], (float) v1[1], t.colors[1], (float) pv1[0], (float) pv1[1], t.colors[2]);
+        GradientPaint gp = new GradientPaint((float) v0[0], (float) v0[1], t.colors[0], (float) pv0[0], (float) pv0[1], TRANSLUCENT_BLACK);
         g2d.setPaint(gp);
         g2d.fill(contour);
 
-        gp = new GradientPaint((float) v2[0], (float) v2[1], t.colors[2], (float) pv2[0], (float) pv2[1], t.colors[0]);
+        gp = new GradientPaint((float) v1[0], (float) v1[1], t.colors[1], (float) pv1[0], (float) pv1[1], TRANSLUCENT_BLACK);
         g2d.setPaint(gp);
         g2d.fill(contour);
 
-        //gp = new GradientPaint((float) v2[0], (float) v2[1], t.colors[2], (float) pv2[0], (float) pv2[1], TRANSLUCENT_BLACK);
-        //g2d.setPaint(gp);
-        //g2d.fill(area);
+        gp = new GradientPaint((float) v2[0], (float) v2[1], t.colors[2], (float) pv2[0], (float) pv2[1], TRANSLUCENT_BLACK);
+        g2d.setPaint(gp);
+        g2d.fill(contour);
 
         g2d.setPaint(oldPaint);
     }
@@ -86,9 +87,6 @@ public final class DrawTools {
      */
     public static final void drawTriangleTexture(final Graphics2D g2d, final BufferedImage image, final double[] ximg, final double[] yimg, final double[] xdest, final double[] ydest, Object key) {
         try {
-            Object oldKey = g2d.getRenderingHint(RenderingHints.KEY_INTERPOLATION);
-            Shape oldClip = g2d.getClip();
-
             double w = image.getWidth();
             double h = image.getHeight();
 
@@ -97,18 +95,28 @@ public final class DrawTools {
             path.lineTo(xdest[1], ydest[1]);
             path.lineTo(xdest[2], ydest[2]);
             path.closePath();
+            Area area = new Area(path);
+            area.add(new Area(stroke.createStrokedShape(path)));
+
+            boolean is1d = is1d(ximg, yimg);
 
             // if we have a 1D texture we must slighlty modified the coordinates to use the algorithm below.
             if (checkSourceCoordinates(ximg, yimg)) {
                 // three coordinates are the same in 1D texture
                 int[] pixels = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
-                int index = (int) Math.round(w * ximg[0]);
+                int index = (int) Math.floor(w * ximg[0]);
+                Color color;
                 if (index >= pixels.length) {
-                    index = pixels.length - 1;
+                    color = new Color(pixels[pixels.length - 1]);
+                } else if (index < 0) {
+                    color = new Color(pixels[0]);
+                } else {
+                    color = new Color(pixels[index]);
                 }
-                Color color = new Color(pixels[index]);
+                //color = Color.PINK;
+
                 g2d.setColor(color);
-                g2d.fill(path);
+                g2d.fill(area);
                 return;
             }
 
@@ -124,21 +132,45 @@ public final class DrawTools {
             transformation.concatenate(fromImg);
             transformation.concatenate(translationImg);
 
-            Area area = new Area(path);
-            area.add(new Area(stroke.createStrokedShape(path)));
-
-            g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, key);
-            g2d.clip(area);
             AffineTransform oldTransform = g2d.getTransform();
-            g2d.drawImage(image, transformation, null);
-            g2d.setTransform(oldTransform);
 
-            if (oldKey != null) {
-                g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, oldKey);
+            // For now we don't enter in this
+            // SVGGraphics2D doesn't handle MultipleGradient :(
+            if (false && is1d && key == RenderingHints.VALUE_INTERPOLATION_BILINEAR) {
+                int[] pixels = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
+                float[] fractions = new float[pixels.length];
+                for (int i = 0; i < fractions.length; i++) {
+                    fractions[i] = (((float) i) / (fractions.length - 1));
+                }
+
+                Color[] colors = new Color[pixels.length];
+                for (int i = 0; i < colors.length; i++) {
+                    colors[i] = new Color(pixels[i]);
+                }
+                LinearGradientPaint gradient = new LinearGradientPaint(new Point2D.Double(0, 0), new Point2D.Double(pixels.length, 0), fractions, colors, MultipleGradientPaint.CycleMethod.NO_CYCLE,  MultipleGradientPaint.ColorSpaceType.SRGB, transformation);
+                Shape oldClip = g2d.getClip();
+                g2d.clip(area);
+                g2d.setPaint(gradient);
+                g2d.fill(area);
+                g2d.setClip(oldClip);
+            } else {
+                clamp(g2d, ximg, yimg, xdest, ydest, transformation, image);
+                Object oldKey = g2d.getRenderingHint(RenderingHints.KEY_INTERPOLATION);
+                g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, key);
+                g2d.setStroke(EMPTYSTROKE);
+                Shape oldClip = g2d.getClip();
+                g2d.clip(area);
+                g2d.drawImage(image, transformation, null);
+                g2d.setClip(oldClip);
+                if (oldKey != null) {
+                    g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, oldKey);
+                }
             }
 
-            g2d.setClip(oldClip);
-        } catch (NoninvertibleTransformException e) { }
+            g2d.setTransform(oldTransform);
+        } catch (NoninvertibleTransformException e) {
+            System.err.println(e);
+        }
     }
 
     /**
@@ -186,9 +218,8 @@ public final class DrawTools {
      * @param x the x-coordinates
      * @param y the y-coordinates
      */
-    private static final boolean checkSourceCoordinates(double[] x, double[] y) {
-        boolean is1D = AbstractDrawable3DObject.isNull(y[0]) && AbstractDrawable3DObject.isNull(y[1]) && AbstractDrawable3DObject.isNull(y[2]);
-        if (is1D) {
+    private static final boolean checkSourceCoordinates(final double[] x, final double[] y) {
+        if (is1d(x, y)) {
             if (!AbstractDrawable3DObject.isEqual(x[0], x[1]) && !AbstractDrawable3DObject.isEqual(x[1], x[2]) && !AbstractDrawable3DObject.isEqual(x[2], x[0])) {
                 y[0] = 1;
             } else if (AbstractDrawable3DObject.isEqual(x[0], x[1]) && !AbstractDrawable3DObject.isEqual(x[1], x[2])) {
@@ -209,6 +240,52 @@ public final class DrawTools {
         return false;
     }
 
+    /**
+     * Check if the triangle in texture is degenerate
+     * @param x x-coordinates
+     * @param y y-coordinates
+     * @return true if 1d
+     */
+    private static final boolean is1d(final double[] x, final double[] y) {
+        return AbstractDrawable3DObject.isNull(y[0]) && AbstractDrawable3DObject.isNull(y[1]) && AbstractDrawable3DObject.isNull(y[2]);
+    }
+
+    private static final void clamp(Graphics2D g2d, double[] ximg, double[] yimg, double[] xdest, double[] ydest, AffineTransform transformation, BufferedImage image) {
+        if (ximg[0] < 0 || ximg[1] < 0 || ximg[2] < 0 || ximg[0] > 1 || ximg[1] > 1 || ximg[2] > 1) {
+            double w = image.getWidth();
+            double h = image.getHeight();
+            int[] pixels = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
+            Path2D.Double path = new Path2D.Double();
+            path.moveTo(w * ximg[0], h * yimg[0]);
+            path.lineTo(w * ximg[1], h * yimg[1]);
+            path.lineTo(w * ximg[2], h * yimg[2]);
+            path.closePath();
+            Area tri = new Area(path);
+            Rectangle2D bounds = tri.getBounds2D();
+
+            if (bounds.getX() < 0) {
+                Area rect = new Area(new Rectangle2D.Double(bounds.getX(), bounds.getY(), -bounds.getX(), bounds.getHeight()));
+                tri.intersect(rect);
+                if (!tri.isEmpty()) {
+                    tri.transform(transformation);
+                    g2d.setColor(new Color(pixels[0]));
+                    g2d.fill(tri);
+                }
+            }
+
+            if (bounds.getX() + bounds.getWidth() > w) {
+                tri = new Area(path);
+                Area rect = new Area(new Rectangle2D.Double(w, bounds.getY(), bounds.getX() + bounds.getWidth() - w, bounds.getHeight()));
+                tri.intersect(rect);
+                if (!tri.isEmpty()) {
+                    tri.transform(transformation);
+                    g2d.setColor(new Color(pixels[pixels.length -1]));
+                    g2d.fill(tri);
+                }
+            }
+        }
+    }
+
     /**
      * Get the projection in 2D of point A on line (BC)
      * @param A the point to project
