@@ -63,6 +63,11 @@ public class Motor3D {
         AbstractDrawable3DObject.resetDefaultPrecedence();
     }
 
+    public void setGraphics(Graphics2D g2d) {
+        this.g2d = g2d;
+        this.textureDrawingTools.setGraphics(g2d);
+    }
+
     public void setAntialiased(boolean aa) {
         if (aa) {
             g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
@@ -128,7 +133,7 @@ public class Motor3D {
      */
     public void draw() {
         Scene.drawRoot(g2d);
-        Scene.clear();
+        Scene.clearAll();
         G2DTextureManager.clear();
     }
 
@@ -182,7 +187,11 @@ public class Motor3D {
         }
 
         if (geometry.getLineDrawingMode() != Geometry.LineDrawingMode.NONE) {
-            addSegments(vertexBuffer, colorBuffer, appearance.getLineColor(), wireIndicesBuffer, geometry.getLineDrawingMode(), G2DStroke.getStroke(appearance));
+            if (appearance.getLineColor() == null) {
+                addSegments(vertexBuffer, colorBuffer, null, wireIndicesBuffer, geometry.getLineDrawingMode(), appearance);
+            } else {
+                addSegments(vertexBuffer, null, appearance.getLineColor(), wireIndicesBuffer, geometry.getLineDrawingMode(), appearance);
+            }
         }
     }
 
@@ -218,16 +227,14 @@ public class Motor3D {
      * @param tri the triangle to add
      */
     private void add(Triangle tri) {
-        if (!is2DView() || tri.isIn2D()) {
-            Vector3d normal = tri.getNormal();
-            if (normal != null) {
-                //normal = transf.projectDirection(normal);
-                if ((mode == FaceCullingMode.CW && normal.getZ() > 0) || (mode == FaceCullingMode.CCW && normal.getZ() < 0) || mode == FaceCullingMode.BOTH) {
-                    Scene.addToRoot(is2DView(), tri);
-                }
-            } else {
+        Vector3d normal = tri.getNormal();
+        if (normal != null) {
+            //normal = transf.projectDirection(normal);
+            if ((mode == FaceCullingMode.CW && normal.getZ() > 0) || (mode == FaceCullingMode.CCW && normal.getZ() < 0) || mode == FaceCullingMode.BOTH) {
                 Scene.addToRoot(is2DView(), tri);
             }
+        } else {
+            Scene.addToRoot(is2DView(), tri);
         }
     }
 
@@ -330,12 +337,13 @@ public class Motor3D {
      * @param drawingMode the drawing mode
      * @param stroke the Stroke to use to draw a segment
      */
-    private void addSegments(FloatBuffer vertices, FloatBuffer colors, Color defaultColor, IntBuffer indices, Geometry.LineDrawingMode drawingMode, Stroke stroke) {
-        Object[] arrays = getArrays(vertices, null, defaultColor, null, indices);
+    private void addSegments(FloatBuffer vertices, FloatBuffer colors, Color defaultColor, IntBuffer indices, Geometry.LineDrawingMode drawingMode, Appearance appearance) {
+        Object[] arrays = getArrays(vertices, colors, defaultColor, null, indices);
         Vector3d[] verticesArray = (Vector3d[]) arrays[0];
         Color[] colorsArray = (Color[]) arrays[1];
         Vector3d[] v;
         Color[] c;
+        double cumLength = 0;
 
         if (verticesArray.length <= 1) {
             return;
@@ -347,8 +355,11 @@ public class Motor3D {
                 v = new Vector3d[] {verticesArray[i], verticesArray[i + 1]};
                 c = new Color[] {colorsArray[i], colorsArray[i + 1]};
                 try {
-                    add(new Segment(v, c, stroke));
-                } catch (InvalidPolygonException e) { }
+                    add(new Segment(v, c, G2DStroke.getStroke(appearance, cumLength)));
+                    cumLength += Segment.getLength(v);
+                } catch (InvalidPolygonException e) {
+                    cumLength = 0;
+                }
             }
             break;
         case SEGMENTS_LOOP :
@@ -356,14 +367,17 @@ public class Motor3D {
                 v = new Vector3d[] {verticesArray[i], verticesArray[i + 1]};
                 c = new Color[] {colorsArray[i], colorsArray[i + 1]};
                 try {
-                    add(new Segment(v, c, stroke));
-                } catch (InvalidPolygonException e) { }
+                    add(new Segment(v, c, G2DStroke.getStroke(appearance, cumLength)));
+                    cumLength += Segment.getLength(v);
+                } catch (InvalidPolygonException e) {
+                    cumLength = 0;
+                }
             }
             int n = verticesArray.length - 1;
             v = new Vector3d[] {verticesArray[n], verticesArray[0]};
             c = new Color[] {colorsArray[n], colorsArray[0]};
             try {
-                add(new Segment(v, c, stroke));
+                add(new Segment(v, c, G2DStroke.getStroke(appearance, cumLength)));
             } catch (InvalidPolygonException e) { }
             break;
         case SEGMENTS :
@@ -372,7 +386,7 @@ public class Motor3D {
                 v = new Vector3d[] {verticesArray[i], verticesArray[i + 1]};
                 c = new Color[] {colorsArray[i], colorsArray[i + 1]};
                 try {
-                    add(new Segment(v, c, stroke));
+                    add(new Segment(v, c, G2DStroke.getStroke(appearance, 0)));
                 } catch (InvalidPolygonException e) { }
             }
             break;
@@ -396,6 +410,7 @@ public class Motor3D {
         Vector3d[] v;
         Color[] c;
         Texture.Filter filter = Texture.Filter.NEAREST;
+
         if (texture != null) {
             filter = texture.getMagnificationFilter();
         }
@@ -404,9 +419,9 @@ public class Motor3D {
         case TRIANGLE_FAN :
             for (int i = 1; i < verticesArray.length - 1; i++) {
                 v = new Vector3d[] {verticesArray[0], verticesArray[i], verticesArray[i + 1]};
-                c = new Color[] {colorsArray[0], colorsArray[i], colorsArray[i + 1]};
                 try {
                     if (image == null) {
+                        c = new Color[] {colorsArray[0], colorsArray[i], colorsArray[i + 1]};
                         add(new Triangle(v, c, null));
                     } else {
                         add(new Triangle(v, new Vector3d[] {textureCoordsArray[0], textureCoordsArray[i], textureCoordsArray[i + 1]}, image, filter));
@@ -415,9 +430,9 @@ public class Motor3D {
             }
             int n = verticesArray.length - 1;
             v = new Vector3d[] {verticesArray[0], verticesArray[n], verticesArray[1]};
-            c = new Color[] {colorsArray[0], colorsArray[n], colorsArray[1]};
             try {
                 if (image == null) {
+                    c = new Color[] {colorsArray[0], colorsArray[n], colorsArray[1]};
                     add(new Triangle(v, c, null));
                 } else {
                     add(new Triangle(v, new Vector3d[] {textureCoordsArray[0], textureCoordsArray[n], textureCoordsArray[1]}, image, filter));
@@ -427,9 +442,9 @@ public class Motor3D {
         case TRIANGLE_STRIP :
             for (int i = 0; i < verticesArray.length - 2; i++) {
                 v = new Vector3d[] {verticesArray[i], verticesArray[i + 1], verticesArray[i + 2]};
-                c = new Color[] {colorsArray[i], colorsArray[i + 1], colorsArray[i + 2]};
                 try {
                     if (image == null) {
+                        c = new Color[] {colorsArray[i], colorsArray[i + 1], colorsArray[i + 2]};
                         add(new Triangle(v, c, null));
                     } else {
                         add(new Triangle(v, new Vector3d[] {textureCoordsArray[i], textureCoordsArray[i + 1], textureCoordsArray[i + 2]}, image, filter));
@@ -441,9 +456,9 @@ public class Motor3D {
         default :
             for (int i = 0; i < verticesArray.length - 2; i += 3) {
                 v = new Vector3d[] {verticesArray[i], verticesArray[i + 1], verticesArray[i + 2]};
-                c = new Color[] {colorsArray[i], colorsArray[i + 1], colorsArray[i + 2]};
                 try {
                     if (image == null) {
+                        c = new Color[] {colorsArray[i], colorsArray[i + 1], colorsArray[i + 2]};
                         add(new Triangle(v, c, null));
                     } else {
                         add(new Triangle(v, new Vector3d[] {textureCoordsArray[i], textureCoordsArray[i + 1], textureCoordsArray[i + 2]}, image, filter));
@@ -504,8 +519,13 @@ public class Motor3D {
     private static final Color[] getMultiColors(final float[] colors) {
         Color[] c = new Color[colors.length / G2DElementsBuffer.ELEMENT_SIZE];
         int j = 0;
+        Color prev = Color.BLACK;
         for (int i = 0; i < c.length; i++) {
             c[i] = new Color(colors[j], colors[j + 1], colors[j + 2], colors[j + 3]);
+            if (prev.equals(c[i])) {
+                c[i] = prev;
+            }
+            prev = c[i];
             j += G2DElementsBuffer.ELEMENT_SIZE;
         }
 
