@@ -18,6 +18,8 @@ import java.awt.geom.Path2D;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.HashSet;
+import java.util.Set;
 
 import org.scilab.forge.scirenderer.tranformations.Vector3d;
 import org.scilab.forge.scirenderer.tranformations.Vector4d;
@@ -39,9 +41,25 @@ public class Segment extends ConvexObject implements Comparable<Segment> {
         this.stroke = stroke;
     }
 
+    public Segment(Vector3d[] vertices, Color[] colors) throws InvalidPolygonException {
+        this(vertices, colors, null);
+    }
+
+    public void setStroke(Stroke stroke) {
+        this.stroke = stroke;
+    }
+
+    public double getLength() {
+        return vertices[0].minus(vertices[1]).getNorm();
+    }
+
+    public static double getLength(Vector3d[] vertices) {
+        return vertices[0].minus(vertices[1]).getNorm();
+    }
+
     public void addConvexObject(ConvexObject co) {
         if (segmentOn == null) {
-            segmentOn = new ArrayList<ConvexObject>();
+            segmentOn = new ArrayList<ConvexObject>(2);
         }
         segmentOn.add(co);
     }
@@ -116,7 +134,7 @@ public class Segment extends ConvexObject implements Comparable<Segment> {
         boolean b = isBehind(vertices[1], np, vv[3]);
 
         if (a && b) {
-            List<ConvexObject> list = new ArrayList<ConvexObject>();
+            List<ConvexObject> list = new ArrayList<ConvexObject>(1);
             list.add(this);
             return list;
         }
@@ -142,7 +160,7 @@ public class Segment extends ConvexObject implements Comparable<Segment> {
         }
 
         try {
-            List<ConvexObject> list = new ArrayList<ConvexObject>();
+            List<ConvexObject> list = new ArrayList<ConvexObject>(1);
             list.add(new Segment(vs, cs, this.stroke));
 
             return list;
@@ -154,7 +172,7 @@ public class Segment extends ConvexObject implements Comparable<Segment> {
     public List<Segment> breakObject(Vector3d p, Vector3d u, Vector3d n) {
         double c = vertices[1].minus(p).scalar(n) / v0.scalar(n);
         if (c > 0 && !isNull(c) && c < 1 && !isEqual(c, 1)) {
-            List<Segment> list = new ArrayList<Segment>();
+            List<Segment> list = new ArrayList<Segment>(2);
             Vector3d q = Vector3d.getBarycenter(vertices[0], vertices[1], c, 1 - c);
             Color color = getColorsBarycenter(colors[0], colors[1], c, 1 - c);
             try {
@@ -164,7 +182,7 @@ public class Segment extends ConvexObject implements Comparable<Segment> {
                 return list;
             } catch (InvalidPolygonException e) { }
         } else {
-            List<Segment> list = new ArrayList<Segment>();
+            List<Segment> list = new ArrayList<Segment>(1);
             try {
                 list.add(new Segment(new Vector3d[]{vertices[0], vertices[1]}, new Color[]{colors[0], colors[1]}, stroke));
 
