@@ -36,13 +36,14 @@ public class Triangle extends ConvexObject {
 
     private static final Color[] COLORS = new Color[]{Color.BLACK, Color.BLACK, Color.BLACK};
     private static final Stroke stroke = new BasicStroke(1f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
+    private static final Stroke EMPTYSTROKE = new BasicStroke(0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
 
     private SpritedRectangle sprite;
     private BufferedImage image;
     private Vector3d[] textureCoords;
     private Texture.Filter filter;
 
-    protected Set<Segment> segments;
+    protected List<Segment> segments;
 
     public Triangle(Vector3d[] vertices, Color[] colors, Vector3d normal) throws InvalidPolygonException {
         super(vertices, colors);
@@ -78,10 +79,11 @@ public class Triangle extends ConvexObject {
     public boolean addSegment(Segment s) {
         if (isSegmentInside(s)) {
             if (segments == null) {
-                segments = new TreeSet<Segment>();
+                segments = new ArrayList<Segment>(3);
             }
             if (segments.contains(s)) {
                 segments.remove(s);
+                s.removeConvexObject(this);
             }
             segments.add(s);
             s.addConvexObject(this);
@@ -99,14 +101,17 @@ public class Triangle extends ConvexObject {
     public void removeSegment(Segment s) {
         if (segments != null) {
             segments.remove(s);
+            s.removeConvexObject(this);
         }
     }
 
     public void replaceSegment(Segment s, List<Segment> segs) {
         segments.remove(s);
+        //s.removeConvexObject(this);
         for (Segment ss : segs) {
             if (segments.contains(ss)) {
                 segments.remove(ss);
+                ss.removeConvexObject(this);
             }
             segments.add(ss);
             ss.addConvexObject(this);
@@ -147,7 +152,7 @@ public class Triangle extends ConvexObject {
             return null;
         }
 
-        List<ConvexObject> list = new ArrayList<ConvexObject>();
+        List<ConvexObject> list = new ArrayList<ConvexObject>(5);
         Vector3d p = Vector3d.getBarycenter(o.vertices[0], o.vertices[1], c, 1 - c);
         Color cc = getColorsBarycenter(o.colors[0], o.colors[1], c, 1 - c);
 
@@ -174,7 +179,11 @@ public class Triangle extends ConvexObject {
     public void draw(Graphics2D g2d) {
         if (sprite != null) {
             Shape oldClip = g2d.getClip();
-            g2d.clip(getProjectedContour());
+            Path2D contour = getProjectedContour();
+            Area area = new Area(contour);
+            // Trick to paint the triangle and its outline
+            area.add(new Area(stroke.createStrokedShape(contour)));
+            g2d.clip(area);//getProjectedContour());
             sprite.draw(g2d);
             g2d.setClip(oldClip);
         } else if (image != null) {
@@ -185,7 +194,7 @@ public class Triangle extends ConvexObject {
             Area area = new Area(contour);
             // Trick to paint the triangle and its outline
             area.add(new Area(stroke.createStrokedShape(contour)));
-
+            g2d.setStroke(EMPTYSTROKE);
             g2d.setColor(colors[0]);
             g2d.fill(area);
         } else {
@@ -208,7 +217,7 @@ public class Triangle extends ConvexObject {
         Vector3d np = new Vector3d(vv);
         int ret = isBehind(np, vv[3]);
         if (ret == 1) {
-            List<ConvexObject> list = new ArrayList<ConvexObject>();
+            List<ConvexObject> list = new ArrayList<ConvexObject>(1);
             list.add(this);
             return list;
         } else if (ret == -1) {
@@ -222,7 +231,7 @@ public class Triangle extends ConvexObject {
         Vector3d p = Vector3d.getBarycenter(u, w, this.v0v1.scalar(this.vertices[0]), -vv[3]);
 
         List<ConvexObject> list1 = breakTriangleOnLine(this, p, np);
-        List<ConvexObject> list = new ArrayList<ConvexObject>();
+        List<ConvexObject> list = new ArrayList<ConvexObject>(3);
         for (ConvexObject o : list1) {
             if (o.isBehind(np, vv[3]) == 1) {
                 list.add(o);
@@ -243,13 +252,13 @@ public class Triangle extends ConvexObject {
         }
 
         Vector3d v0v1v2 = Vector3d.product(v0v1, v2);
-        double a = v0v1v2.scalar(v1);
-        if (a < 0 && !isNull(a)) {
+        double a = -v0v1v2.scalar(v1);
+        if (a < 0 || isNull(a)) {
             return false;
         }
 
-        double b = -v0v1v2.scalar(v0);
-        if (b < 0 && !isNull(b)) {
+        double b = v0v1v2.scalar(v0);
+        if (b < 0 || isNull(b)) {
             return false;
         }
 
@@ -331,7 +340,7 @@ public class Triangle extends ConvexObject {
             return null;
         }
 
-        List<ConvexObject> list = new ArrayList<ConvexObject>();
+        List<ConvexObject> list = new ArrayList<ConvexObject>(2);
         Vector3d p = Vector3d.getBarycenter(s.vertices[0], s.vertices[1], c, 1 - c);
         Color cc = getColorsBarycenter(s.colors[0], s.colors[1], c, 1 - c);
 
@@ -361,7 +370,7 @@ public class Triangle extends ConvexObject {
         Vector3d v2 = t.vertices[2].minus(t.vertices[1]);
         double c = t.vertices[2].minus(p).scalar(n) / v2.scalar(n);
 
-        List<ConvexObject> list = new ArrayList<ConvexObject>();
+        List<ConvexObject> list = new ArrayList<ConvexObject>(3);
         int i = -1;
         int j = -1;
         int k = -1;
