@@ -26,6 +26,7 @@ import java.awt.Dimension;
 import java.awt.Graphics2D;
 import java.awt.RenderingHints;
 import java.awt.font.TextLayout;
+import java.awt.geom.Ellipse2D;
 import java.awt.geom.Rectangle2D;
 
 /**
@@ -37,7 +38,7 @@ import java.awt.geom.Rectangle2D;
  */
 public class G2DTextureDrawingTools implements TextureDrawingTools {
 
-    private final Graphics2D g2d;
+    private Graphics2D g2d;
 
     /**
      * Default constructor.
@@ -49,6 +50,10 @@ public class G2DTextureDrawingTools implements TextureDrawingTools {
         this.g2d = g2d;
     }
 
+    public void setGraphics(Graphics2D g2d) {
+        this.g2d = g2d;
+    }
+
     public void accept(Texture texture) {
         G2DTextureManager.G2DTexture t = (G2DTextureManager.G2DTexture) texture;
         TextureDrawer drawer = t.getDrawer();
@@ -67,11 +72,37 @@ public class G2DTextureDrawingTools implements TextureDrawingTools {
             g2d.translate(width / 2.0, height / 2.0);
         }
 
+        boolean aa = g2d.getRenderingHint(RenderingHints.KEY_ANTIALIASING) == RenderingHints.VALUE_ANTIALIAS_ON;
+        if (!aa) {
+            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
+        }
         textureDrawer.draw(this);
+
+        if (!aa) {
+            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
+        }
+    }
+
+    @Override
+    public void drawPlus(int size, Appearance appearance) {
+        if (size > 1) {
+            int r = size / 2;
+            int[] coords1 = new int[]{-r, 0, r, 0};
+            int[] coords2 = new int[]{0, -r, 0, r};
+
+            drawPolyline(coords1, appearance);
+            drawPolyline(coords2, appearance);
+        } else {
+            fillDisc(0, 0, 1, appearance.getLineColor());
+        }
     }
 
     @Override
     public void drawPolyline(int[] coordinates, Appearance appearance) {
+        if (coordinates.length == 2) {
+
+        }
+
         int nbPoint = coordinates.length / 2;
 
         int[] xCoordinates = new int[nbPoint];
@@ -85,7 +116,7 @@ public class G2DTextureDrawingTools implements TextureDrawingTools {
         }
 
         g2d.setColor(appearance.getLineColor());
-        g2d.setStroke(G2DStroke.getStroke(appearance));
+        g2d.setStroke(G2DStroke.getStroke(appearance, 0));
 
         g2d.drawPolyline(xCoordinates, yCoordinates, nbPoint);
     }
@@ -123,17 +154,17 @@ public class G2DTextureDrawingTools implements TextureDrawingTools {
     @Override
     public void drawCircle(int x, int y, int diameter, Appearance appearance) {
         g2d.setColor(appearance.getLineColor());
-        g2d.setStroke(G2DStroke.getStroke(appearance));
-        int r = diameter / 2;
-        g2d.drawOval(x - r, y - r, diameter, diameter);
+        g2d.setStroke(G2DStroke.getStroke(appearance, 0));
+        double r = ((double) diameter) / 2;
+        g2d.draw(new Ellipse2D.Double(x - r, y - r, diameter, diameter));
     }
 
     @Override
     public void fillDisc(int x, int y, int diameter, Color color) {
         if (color.getAlphaAsFloat() != 0) {
             g2d.setColor(color);
-            int r = diameter / 2;
-            g2d.fillOval(x - r, y - r, diameter, diameter);
+            double r = ((double) diameter) / 2;
+            g2d.fill(new Ellipse2D.Double(x - r, y - r, diameter, diameter));
         }
     }
 
@@ -154,8 +185,8 @@ public class G2DTextureDrawingTools implements TextureDrawingTools {
             g2d.setColor(textEntity.getTextColor());
             TextLayout textLayout = new TextLayout(textEntity.getText(), textEntity.getFont(), g2d.getFontRenderContext());
             Rectangle2D bounds = textLayout.getBounds();
-	    g2d.setFont(textEntity.getFont());
-	    g2d.drawString(textEntity.getText(), (float) (x - bounds.getX()), (float) (y - bounds.getY()));
+            g2d.setFont(textEntity.getFont());
+            g2d.drawString(textEntity.getText(), (float) (x - bounds.getX()), (float) (y - bounds.getY()));
         }
     }
 
