@@ -11,6 +11,7 @@
 
 package org.scilab.forge.scirenderer.implementation.g2d.texture;
 
+import org.scilab.forge.scirenderer.DrawingTools;
 import org.scilab.forge.scirenderer.SciRendererException;
 import org.scilab.forge.scirenderer.implementation.g2d.G2DCanvas;
 import org.scilab.forge.scirenderer.implementation.g2d.G2DDrawingTools;
@@ -65,14 +66,18 @@ public class G2DTextureManager implements TextureManager {
     }
 
     @Override
-    public void dispose(Texture texture) { }
+    public void dispose(Texture texture) {
+        if (texture instanceof G2DTexture) {
+            ((G2DTexture) texture).dispose();
+        }
+    }
 
     /**
      * Inner class for {@link Texture} implementation.
      */
     public class G2DTexture extends AbstractTexture implements Texture {
 
-        private BufferedImage image;
+        private ImageBuffer image;
         private TextureDrawer drawer;
 
         /**
@@ -80,6 +85,10 @@ public class G2DTextureManager implements TextureManager {
          */
         public G2DTexture() { }
 
+        public void dispose() {
+            images.remove(image);
+        }
+
         @Override
         public void setDrawer(TextureDrawer drawer) {
             this.drawer = drawer;
@@ -113,10 +122,10 @@ public class G2DTextureManager implements TextureManager {
                     ib2 = ib1;
                 }
 
-                image = ib2.getImage();
+                image = ib2;
             }
 
-            return image;
+            return image.getImage();
         }
     }
 
