@@ -19,9 +19,11 @@ import org.scilab.forge.scirenderer.implementation.jogl.renderer.JoGLRendererMan
 import org.scilab.forge.scirenderer.implementation.jogl.texture.JoGLTextureManager;
 import org.scilab.forge.scirenderer.picking.PickingManager;
 
+import com.jogamp.opengl.util.awt.ImageUtil;
 import com.jogamp.opengl.util.awt.Screenshot;
 import java.awt.Dimension;
 import java.awt.image.BufferedImage;
+import java.lang.reflect.InvocationTargetException;
 import java.util.concurrent.Semaphore;
 import javax.media.opengl.DebugGL2;
 import javax.media.opengl.GL2;
@@ -34,6 +36,7 @@ import javax.media.opengl.GLException;
 import javax.media.opengl.GLPbuffer;
 import javax.media.opengl.GLProfile;
 import javax.swing.SwingUtilities;
+import jogamp.opengl.GLContextImpl;
 
 /**
  * JoGL implementation of a Canvas.
@@ -72,6 +75,8 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
     private final CanvasAnimator canvasAnimator;
     private boolean isOffscreen;
     private DebugGL2 debug;
+    private boolean isValid = true;
+    private boolean displayFinished;
 
 
     /** The current mainDrawer. */
@@ -221,7 +226,7 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
      * @return an image
      */
     public BufferedImage getImage() {
-        while (!canvasAnimator.isDrawFinished()) {
+        while (!canvasAnimator.isDrawFinished() || !displayFinished) {
             try {
                 Thread.sleep(10);
             } catch (InterruptedException e) {
@@ -229,14 +234,37 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
             }
         }
 
-        synchronized (canvasAnimator) {
-            GLContext context = autoDrawable.getContext();
-            context.makeCurrent();
-            BufferedImage image = Screenshot.readToBufferedImage(autoDrawable.getWidth(), autoDrawable.getHeight());
-            context.release();
+        final BufferedImage[] image = new BufferedImage[1];
+        final GLContext context = autoDrawable.getContext();
 
-            return image;
+        try {
+            SwingUtilities.invokeAndWait(new Runnable() {
+                    public void run() {
+                        context.makeCurrent();
+                        image[0] = Screenshot.readToBufferedImage(autoDrawable.getWidth(), autoDrawable.getHeight());
+                        context.release();
+                    }
+                });
+        } catch (InterruptedException e) {
+
+        } catch (InvocationTargetException e) {
+            System.err.println(e);
+            e.printStackTrace();
         }
+
+        boolean needFlip;
+        try {
+            needFlip = !((GLContextImpl) context).offscreenImageNeedsVerticalFlip();
+        } catch (GLException e) {
+            needFlip = false;
+        }
+
+        if (needFlip) {
+            // Need to flip image when Windows uses an OpenGL software renderer
+            ImageUtil.flipImageVertically(image[0]);
+        }
+
+        return image[0];
     }
 
     /**
@@ -247,6 +275,7 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
             ((GLPbuffer) autoDrawable).destroy();
         }
         try {
+            isValid = false;
             canvasAnimator.finalize();//Thread.dumpStack();
         } catch (Throwable e) {
             // TODO: handle exception
@@ -269,49 +298,53 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
     // Implementation of function from GLEventListener.
     @Override
     public void display(GLAutoDrawable glAutoDrawable) {
-        GL2 gl = getGl().getGL2();
-        buffersManager.glSynchronize(gl);
-        rendererManager.glSynchronize(gl);
-        drawingTools.glSynchronize(gl);
+        if (isValid) {
+            displayFinished = false;
+            GL2 gl = getGl().getGL2();
+            buffersManager.glSynchronize(gl);
+            rendererManager.glSynchronize(gl);
+            drawingTools.glSynchronize(gl);
 
-        if (mainDrawer != null) {
-            gl.glEnable(GL2.GL_DEPTH_TEST);
-            gl.glDepthFunc(GL2.GL_LEQUAL); // Set to less equal to allow last drawn object to be on the top.
+            if (mainDrawer != null) {
+                gl.glEnable(GL2.GL_DEPTH_TEST);
+                gl.glDepthFunc(GL2.GL_LEQUAL); // Set to less equal to allow last drawn object to be on the top.
 
-            if ((antiAliasingLevel > 0) && (antiAliasingLevel < 5) && (drawingTools.getGLCapacity().isAccumulationBufferPresent())) {
-                org.scilab.forge.scirenderer.renderer.Renderer renderer = rendererManager.createRenderer();
-                renderer.setDrawer(mainDrawer);
+                if ((antiAliasingLevel > 0) && (antiAliasingLevel < 5) && (drawingTools.getGLCapacity().isAccumulationBufferPresent())) {
+                    org.scilab.forge.scirenderer.renderer.Renderer renderer = rendererManager.createRenderer();
+                    renderer.setDrawer(mainDrawer);
 
-                double[][] jitter = ANTI_ALIASING_JITTER[antiAliasingLevel];
-                Dimension dimension = getDimension();
-                gl.glClear(GL2.GL_ACCUM_BUFFER_BIT);
-                for (double[] aJitter : jitter) {
-                    drawingTools.glSynchronize(gl);
+                    double[][] jitter = ANTI_ALIASING_JITTER[antiAliasingLevel];
+                    Dimension dimension = getDimension();
+                    gl.glClear(GL2.GL_ACCUM_BUFFER_BIT);
+                    for (double[] aJitter : jitter) {
+                        drawingTools.glSynchronize(gl);
 
+                        gl.glMatrixMode(GL2.GL_PROJECTION);
+                        gl.glLoadIdentity();
+                        gl.glTranslated(aJitter[0] / dimension.getWidth(), aJitter[1] / dimension.getHeight(), 0.);
+
+                        gl.glClear(GL2.GL_DEPTH_BUFFER_BIT);
+                        rendererManager.draw(drawingTools, renderer);
+                        //mainDrawer.draw(drawingTools);
+                        gl.glReadBuffer(GL2.GL_BACK);
+                        gl.glAccum(GL2.GL_ACCUM, 1f / jitter.length);
+                    }
+
+                    rendererManager.dispose(drawingTools, renderer);
+
+                    gl.glDrawBuffer(GL2.GL_BACK);
+                    gl.glAccum(GL2.GL_RETURN, 1.0f);
+                } else {
                     gl.glMatrixMode(GL2.GL_PROJECTION);
                     gl.glLoadIdentity();
-                    gl.glTranslated(aJitter[0] / dimension.getWidth(), aJitter[1] / dimension.getHeight(), 0.);
-
                     gl.glClear(GL2.GL_DEPTH_BUFFER_BIT);
-                    rendererManager.draw(drawingTools, renderer);
-                    //mainDrawer.draw(drawingTools);
-                    gl.glReadBuffer(GL2.GL_BACK);
-                    gl.glAccum(GL2.GL_ACCUM, 1f / jitter.length);
+                    mainDrawer.draw(drawingTools);
                 }
-
-                rendererManager.dispose(drawingTools, renderer);
-
-                gl.glDrawBuffer(GL2.GL_BACK);
-                gl.glAccum(GL2.GL_RETURN, 1.0f);
-            } else {
-                gl.glMatrixMode(GL2.GL_PROJECTION);
-                gl.glLoadIdentity();
-                gl.glClear(GL2.GL_DEPTH_BUFFER_BIT);
-                mainDrawer.draw(drawingTools);
             }
-        }
 
-        pickingManager.glConsume(drawingTools);
+            pickingManager.glConsume(drawingTools);
+            displayFinished = true;
+        }
     }
 
     @Override
@@ -355,7 +388,7 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
             super.finalize();
         }
 
-        public boolean isDrawFinished() {
+        public synchronized boolean isDrawFinished() {
             return semaphore.availablePermits() == 0;
         }
 
