@@ -11,14 +11,22 @@
 
 package org.scilab.forge.scirenderer.implementation.jogl;
 
-import javax.media.opengl.GLAutoDrawable;
-
 import org.scilab.forge.scirenderer.Canvas;
 
+import javax.media.opengl.GLAutoDrawable;
+
 /**
  * @author Pierre Lando
  */
-public class JoGLCanvasFactory {
+public final class JoGLCanvasFactory {
+
+    /**
+     * Private constructor.
+     * This is an utility class.
+     */
+    private JoGLCanvasFactory() {
+
+    }
 
     /**
      * Create a canvas from an auto drawable object.
@@ -28,4 +36,14 @@ public class JoGLCanvasFactory {
     public static Canvas createCanvas(final GLAutoDrawable autoDrawable) {
         return new JoGLCanvas(autoDrawable);
     }
+
+    /**
+     * Create a canvas for an offscreen rendering.
+     * @param width the width
+     * @param height the height
+     * @return a canvas.
+     */
+    public static Canvas createCanvas(final int width, final int height) {
+        return new JoGLCanvas(width, height);
+    }
 }
