@@ -20,26 +20,21 @@ import javax.media.opengl.GL2;
  */
 public class JoGLCapacity {
 
-    /**
-     * The actual aliased point size range.
-     */
+    /** The actual aliased point size range. */
     private final float[] aliasedPointSizeRange = new float[] {0, 0};
 
-    /**
-     * The actual light number.
-     */
+    /** The actual light number. */
     private final  int[] lightNumber = new int[] {8};
 
-    /**
-     * The actual light number.
-     */
+    /** The actual light number. */
     private final int[] clippingPlaneNumber = new int[] {6};
 
-    /**
-     * Maximum texture size.
-     */
+    /** Maximum texture size. */
     private final int[] textureSize = new int[] {64};
 
+    /** Accumulation buffer presence */
+    private boolean accumulationBufferPresent = false;
+
     /**
      * Default constructor.
      * The constructor is package because, only {@link JoGLDrawingTools} use this class.
@@ -56,6 +51,17 @@ public class JoGLCapacity {
         gl.glGetIntegerv(GL2.GL_MAX_CLIP_PLANES, clippingPlaneNumber, 0);
         gl.glGetIntegerv(GL2.GL_MAX_LIGHTS, lightNumber, 0);
         gl.glGetIntegerv(GL2.GL_MAX_TEXTURE_SIZE, textureSize, 0);
+
+        int[] nbAccumulationBits = {0, 0, 0};
+        gl.glGetIntegerv(GL2.GL_ACCUM_RED_BITS, nbAccumulationBits, 0);
+        gl.glGetIntegerv(GL2.GL_ACCUM_GREEN_BITS, nbAccumulationBits, 1);
+        gl.glGetIntegerv(GL2.GL_ACCUM_BLUE_BITS, nbAccumulationBits, 2);
+        if (nbAccumulationBits[0] == 0 || nbAccumulationBits[1] == 0 || nbAccumulationBits[2] == 0) {
+            // accumulation buffers not there
+            accumulationBufferPresent = false;
+        } else {
+            accumulationBufferPresent = true;
+        }
     }
 
     /**
@@ -82,4 +88,8 @@ public class JoGLCapacity {
     public int getMaximumTextureSize() {
         return textureSize[0];
     }
+
+    public boolean isAccumulationBufferPresent() {
+        return accumulationBufferPresent;
+    }
 }
