@@ -61,20 +61,20 @@ public final class JoGLShapeDrawer {
         GL2 gl = drawingTools.getGl().getGL2();
         gl.glFrontFace(GL2.GL_CCW);
         switch (geometry.getFaceCullingMode()) {
-            case CW:
-                gl.glEnable(GL2.GL_CULL_FACE);
-                gl.glCullFace(GL2.GL_FRONT);
-                break;
-            case CCW:
-                gl.glEnable(GL2.GL_CULL_FACE);
-                gl.glCullFace(GL2.GL_BACK);
-                break;
-            case BOTH:
-                gl.glDisable(GL2.GL_CULL_FACE);
-                break;
-            default:
-                gl.glDisable(GL2.GL_CULL_FACE);
-                break;
+        case CW:
+            gl.glEnable(GL2.GL_CULL_FACE);
+            gl.glCullFace(GL2.GL_FRONT);
+            break;
+        case CCW:
+            gl.glEnable(GL2.GL_CULL_FACE);
+            gl.glCullFace(GL2.GL_BACK);
+            break;
+        case BOTH:
+            gl.glDisable(GL2.GL_CULL_FACE);
+            break;
+        default:
+            gl.glDisable(GL2.GL_CULL_FACE);
+            break;
         }
 
         if (drawingTools.getCanvas().getJoGLParameters().useVBO()) {
@@ -107,9 +107,13 @@ public final class JoGLShapeDrawer {
 
         buffersManager.bindNormalsBuffer(gl, geometry.getNormals());
 
-        if ((texture != null) && (geometry.getTextureCoordinates() != null)) {
-            drawingTools.bind(texture);
-            buffersManager.bindTextureCoordinatesBuffer(gl, geometry.getTextureCoordinates());
+        if (texture != null && geometry.getTextureCoordinates() != null) {
+            synchronized (texture) {
+                if (texture.isValid()) {
+                    drawingTools.bind(texture);
+                    buffersManager.bindTextureCoordinatesBuffer(gl, geometry.getTextureCoordinates());
+                }
+            }
         } else {
             buffersManager.bindColorsBuffer(gl, geometry.getColors());
         }
@@ -126,11 +130,15 @@ public final class JoGLShapeDrawer {
         if (geometry.getFillDrawingMode() != Geometry.FillDrawingMode.NONE) {
             if (indices != null) {
                 int indicesSize = buffersManager.bindIndicesBuffer(gl, indices);
-                gl.glDrawElements(getGlMode(geometry.getFillDrawingMode()), indicesSize, GL2.GL_UNSIGNED_INT, 0);
-                gl.glBindBuffer(GL2.GL_ELEMENT_ARRAY_BUFFER, 0);
+                if (indicesSize > 0) {
+                    gl.glDrawElements(getGlMode(geometry.getFillDrawingMode()), indicesSize, GL2.GL_UNSIGNED_INT, 0);
+                    gl.glBindBuffer(GL2.GL_ELEMENT_ARRAY_BUFFER, 0);
+                }
             } else {
                 int count = geometry.getVertices().getSize();
-                gl.glDrawArrays(getGlMode(geometry.getFillDrawingMode()), 0, count);
+                if (count > 0) {
+                    gl.glDrawArrays(getGlMode(geometry.getFillDrawingMode()), 0, count);
+                }
             }
         }
 
@@ -144,7 +152,7 @@ public final class JoGLShapeDrawer {
         gl.glDisable(GL2.GL_TEXTURE_2D);
 
         if (geometry.getLineDrawingMode() != Geometry.LineDrawingMode.NONE) {
-            if ((appearance.getLineColor() != null) || (geometry.getColors() != null)) {
+            if (appearance.getLineColor() != null || geometry.getColors() != null) {
                 GLShortCuts.useLineAppearance(gl, appearance);
                 if (appearance.getLineColor() == null) {
                     buffersManager.bindColorsBuffer(gl, geometry.getColors());
@@ -152,11 +160,15 @@ public final class JoGLShapeDrawer {
 
                 if (geometry.getWireIndices() != null) {
                     int edgesIndicesSize = buffersManager.bindIndicesBuffer(gl, geometry.getWireIndices());
-                    gl.glDrawElements(getGlMode(geometry.getLineDrawingMode()), edgesIndicesSize, GL2.GL_UNSIGNED_INT, 0);
-                    gl.glBindBuffer(GL2.GL_ELEMENT_ARRAY_BUFFER, 0);
+                    if (edgesIndicesSize > 0) {
+                        gl.glDrawElements(getGlMode(geometry.getLineDrawingMode()), edgesIndicesSize, GL2.GL_UNSIGNED_INT, 0);
+                        gl.glBindBuffer(GL2.GL_ELEMENT_ARRAY_BUFFER, 0);
+                    }
                 } else {
                     int count = geometry.getVertices().getSize();
-                    gl.glDrawArrays(getGlMode(geometry.getLineDrawingMode()), 0, count);
+                    if (count > 0) {
+                        gl.glDrawArrays(getGlMode(geometry.getLineDrawingMode()), 0, count);
+                    }
                 }
 
                 gl.glDisableClientState(GL2.GL_COLOR_ARRAY);
@@ -175,7 +187,7 @@ public final class JoGLShapeDrawer {
      */
     private void directDrawing(
         JoGLDrawingTools drawingTools, Geometry geometry, Appearance appearance
-    ) throws SciRendererException {
+        ) throws SciRendererException {
         final double sfactor;
         final double tfactor;
         final float[] fbuffer = new float[4];
@@ -205,10 +217,12 @@ public final class JoGLShapeDrawer {
         Texture texture = appearance.getTexture();
         FloatBuffer textureCoordinatesBuffer;
         if (texture != null && geometry.getTextureCoordinates() != null) {
-            drawingTools.bind(texture);
-            textureCoordinatesBuffer = geometry.getTextureCoordinates().getData();
-            sfactor = texture.getSScaleFactor();
-            tfactor = texture.getTScaleFactor();
+            synchronized (texture) {
+                drawingTools.bind(texture);
+                textureCoordinatesBuffer = geometry.getTextureCoordinates().getData();
+                sfactor = texture.getSScaleFactor();
+                tfactor = texture.getTScaleFactor();
+            }
         } else {
             textureCoordinatesBuffer = null;
             sfactor = 1;
@@ -307,7 +321,6 @@ public final class JoGLShapeDrawer {
                         if ((index * elementsSize) < vertexBuffer.limit()) {
                             vertexBuffer.position(index * elementsSize);
                             gl.glVertex4fv(vertexBuffer);
-
                         }
                     }
                 } else {
@@ -354,14 +367,14 @@ public final class JoGLShapeDrawer {
      */
     private int getGlMode(Geometry.FillDrawingMode drawingMode) {
         switch (drawingMode) {
-            case TRIANGLE_FAN:
-                return GL2.GL_TRIANGLE_FAN;
-            case TRIANGLE_STRIP:
-                return GL2.GL_TRIANGLE_STRIP;
-            case TRIANGLES:
-                return GL2.GL_TRIANGLES;
-            default:
-                return GL2.GL_TRIANGLES;
+        case TRIANGLE_FAN:
+            return GL2.GL_TRIANGLE_FAN;
+        case TRIANGLE_STRIP:
+            return GL2.GL_TRIANGLE_STRIP;
+        case TRIANGLES:
+            return GL2.GL_TRIANGLES;
+        default:
+            return GL2.GL_TRIANGLES;
         }
     }
 
@@ -372,14 +385,14 @@ public final class JoGLShapeDrawer {
      */
     private int getGlMode(Geometry.LineDrawingMode drawingMode) {
         switch (drawingMode) {
-            case SEGMENTS:
-                return GL2.GL_LINES;
-            case SEGMENTS_LOOP:
-                return GL2.GL_LINE_LOOP;
-            case SEGMENTS_STRIP:
-                return GL2.GL_LINE_STRIP;
-            default:
-                return GL2.GL_LINES;
+        case SEGMENTS:
+            return GL2.GL_LINES;
+        case SEGMENTS_LOOP:
+            return GL2.GL_LINE_LOOP;
+        case SEGMENTS_STRIP:
+            return GL2.GL_LINE_STRIP;
+        default:
+            return GL2.GL_LINES;
         }
     }
 }
