@@ -57,6 +57,15 @@ public class JoGLRendererManager implements RendererManager {
         }
     }
 
+    public void dispose(JoGLDrawingTools drawingTools, Renderer renderer) {
+        if ((renderer != null) && (renderer instanceof JoGLRenderer)) {
+            JoGLRenderer localRenderer = (JoGLRenderer) renderer;
+            localRenderer.dispose(drawingTools.getGl().getGL2());
+            rendererSet.remove(localRenderer);
+            deadRendererSet.push(localRenderer);
+        }
+    }
+
     /**
      * Perform a draw with the given renderer to the given canvas..
      * @param drawingTools the given drawing tools.
