@@ -27,6 +27,8 @@ import org.scilab.forge.scirenderer.tranformations.Transformation;
 import org.scilab.forge.scirenderer.tranformations.TransformationManager;
 import org.scilab.forge.scirenderer.tranformations.Vector3d;
 
+import javax.media.opengl.GLContext;
+
 import javax.media.opengl.GL2;
 import java.awt.AlphaComposite;
 import java.awt.Dimension;
@@ -40,7 +42,9 @@ import java.util.Set;
  * @author Pierre Lando
  */
 public class JoGLTextureManager implements TextureManager {
+
     private final Set<JoGLTexture> allTextures = new HashSet<JoGLTexture>();
+    private final Set<JoGLTexture> toDispose = new HashSet<JoGLTexture>();
     JoGLCanvas canvas;
 
     public JoGLTextureManager(JoGLCanvas canvas) {
@@ -127,8 +131,10 @@ public class JoGLTextureManager implements TextureManager {
     @Override
     public void dispose(Texture texture) {
         if ((texture instanceof JoGLTexture) && (allTextures.contains((JoGLTexture) texture))) {
-            ((JoGLTexture) texture).dispose();
             allTextures.remove((JoGLTexture) texture);
+            synchronized (toDispose) {
+                toDispose.add((JoGLTexture) texture);
+            }
         }
     }
 
@@ -137,7 +143,6 @@ public class JoGLTextureManager implements TextureManager {
      */
     public class JoGLTexture extends AbstractTexture implements Texture {
         private com.jogamp.opengl.util.texture.Texture[] textures;
-        private JoGLDrawingTools drawingTools;
         private int wCuts;
         private int hCuts;
         private double sfactor = 1;
@@ -149,33 +154,30 @@ public class JoGLTextureManager implements TextureManager {
         public JoGLTexture() {
         }
 
-        public void dispose() {
-            if (drawingTools != null) {
-                final GL2 gl = drawingTools.getGl().getGL2();
-                if (textures != null) {
-                    for (com.jogamp.opengl.util.texture.Texture t : textures) {
-                        t.destroy(gl);
-                    }
-                }
-            }
-        }
-
         /**
          * Bind the texture in the OpenGl context.
          * @param drawingTools current drawing tools.
          * @throws SciRendererException if the texture is invalid.
          */
-        public void bind(JoGLDrawingTools drawingTools) throws SciRendererException {
-            this.drawingTools = drawingTools;
+        public synchronized void bind(JoGLDrawingTools drawingTools) throws SciRendererException {
             GL2 gl = drawingTools.getGl().getGL2();
             if (isValid()) {
                 checkData(drawingTools);
                 if (textures.length == 1) {
                     gl.glEnable(GL2.GL_TEXTURE_2D);
+                    gl.glEnable(GL2.GL_BLEND);
+                    gl.glBlendFunc(GL2.GL_ONE, GL2.GL_ONE_MINUS_SRC_ALPHA);
+                    gl.glTexEnvi(GL2.GL_TEXTURE_ENV, GL2.GL_TEXTURE_ENV_MODE, GL2.GL_REPLACE);
+
                     textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_MAG_FILTER, getAsGLFilter(getMagnificationFilter(), false));
                     textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_MIN_FILTER, getAsGLFilter(getMinifyingFilter(), false));
-                    textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, getAsGLWrappingMode(getSWrappingMode()));
-                    textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, getAsGLWrappingMode(getTWrappingMode()));
+                    if (gl.isNPOTTextureAvailable()) {
+                        textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, getAsGLWrappingMode(getSWrappingMode()));
+                        textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, getAsGLWrappingMode(getTWrappingMode()));
+                    } else {
+                        textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, GL2.GL_CLAMP_TO_EDGE);
+                        textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, GL2.GL_CLAMP_TO_EDGE);
+                    }
                     gl.glTexEnvi(GL2.GL_TEXTURE_ENV, GL2.GL_TEXTURE_ENV_MODE, GL2.GL_REPLACE);
                     textures[0].bind(gl);
 
@@ -197,7 +199,17 @@ public class JoGLTextureManager implements TextureManager {
          * @param drawingTools the drawing tools.
          * @throws SciRendererException if the texture is too big.
          */
-        private void checkData(JoGLDrawingTools drawingTools) throws SciRendererException {
+        private synchronized void checkData(JoGLDrawingTools drawingTools) throws SciRendererException {
+            synchronized (toDispose) {
+                if (!toDispose.isEmpty()) {
+                    final GL2 gl = drawingTools.getGl().getGL2();
+                    for (JoGLTexture jt : toDispose) {
+                        jt.releaseTextures(gl);
+                    }
+                    toDispose.clear();
+                }
+            }
+
             if (isValid() && !upToDate) {
                 GL2 gl = drawingTools.getGl().getGL2();
                 releaseTextures(gl);
@@ -268,8 +280,13 @@ public class JoGLTextureManager implements TextureManager {
                 textures[k].enable(gl);
                 textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_MAG_FILTER, getAsGLFilter(getMagnificationFilter(), false));
                 textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_MIN_FILTER, getAsGLFilter(getMinifyingFilter(), false));
-                textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, getAsGLWrappingMode(getSWrappingMode()));
-                textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, getAsGLWrappingMode(getTWrappingMode()));
+                if (gl.isNPOTTextureAvailable()) {
+                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, getAsGLWrappingMode(getSWrappingMode()));
+                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, getAsGLWrappingMode(getTWrappingMode()));
+                } else {
+                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, GL2.GL_CLAMP_TO_EDGE);
+                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, GL2.GL_CLAMP_TO_EDGE);
+                }
                 textures[k].bind(gl);
             }
         }
@@ -335,29 +352,31 @@ public class JoGLTextureManager implements TextureManager {
             final Dimension textureSize = getDataProvider().getTextureSize();
             final GL2 gl = drawingTools.getGl().getGL2();
 
-            int k = 0;
-            for (int i = 0; i < wCuts; i++) {
-                for (int j = 0; j < hCuts; j++) {
-                    final int x = i * maxSize;
-                    final int y = j * maxSize;
-                    final int width = getSubTextureSize((i == (wCuts - 1)), textureSize.width, maxSize);
-                    final int height = getSubTextureSize((j == (hCuts - 1)), textureSize.height, maxSize);
+            if (textureSize != null) {
+                int k = 0;
+                for (int i = 0; i < wCuts; i++) {
+                    for (int j = 0; j < hCuts; j++) {
+                        final int x = i * maxSize;
+                        final int y = j * maxSize;
+                        final int width = getSubTextureSize((i == (wCuts - 1)), textureSize.width, maxSize);
+                        final int height = getSubTextureSize((j == (hCuts - 1)), textureSize.height, maxSize);
 
-                    // if the texture has been transformed in a power-of-two texture we need to have the correct coords.
-                    final TextureCoords coords = textures[k].getSubImageTexCoords(0, 0, width, height);
+                        // if the texture has been transformed in a power-of-two texture we need to have the correct coords.
+                        final TextureCoords coords = textures[k].getSubImageTexCoords(0, 0, width, height);
 
-                    gl.glBegin(GL2.GL_QUADS);
-                    gl.glTexCoord2f(coords.left(), coords.bottom());
-                    gl.glVertex2f(x, y);
-                    gl.glTexCoord2f(coords.right(), coords.bottom());
-                    gl.glVertex2f(x + width, y);
-                    gl.glTexCoord2f(coords.right(), coords.top());
-                    gl.glVertex2f(x + width, y + height);
-                    gl.glTexCoord2f(coords.left(), coords.top());
-                    gl.glVertex2f(x, y + height);
-                    gl.glEnd();
+                        gl.glBegin(GL2.GL_QUADS);
+                        gl.glTexCoord2f(coords.left(), coords.bottom());
+                        gl.glVertex2f(x, y);
+                        gl.glTexCoord2f(coords.right(), coords.bottom());
+                        gl.glVertex2f(x + width, y);
+                        gl.glTexCoord2f(coords.right(), coords.top());
+                        gl.glVertex2f(x + width, y + height);
+                        gl.glTexCoord2f(coords.left(), coords.top());
+                        gl.glVertex2f(x, y + height);
+                        gl.glEnd();
 
-                    k++;
+                        k++;
+                    }
                 }
             }
         }
@@ -395,7 +414,7 @@ public class JoGLTextureManager implements TextureManager {
         private int getAsGLWrappingMode(Texture.Wrap wrappingMode) {
             switch (wrappingMode) {
             case CLAMP:
-                return GL2.GL_CLAMP;
+                return GL2.GL_CLAMP_TO_EDGE;
             case REPEAT:
                 return GL2.GL_REPEAT;
             default:
