@@ -1,6 +1,7 @@
 /*
  * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
  * Copyright (C) 2009-2011 - DIGITEO - Pierre Lando
+ * Copyright (C) 2013 - Scilab Enterprises - Calixte DENIZET
  *
  * This file must be used under the terms of the CeCILL.
  * This source file is licensed as described in the file COPYING, which
@@ -110,7 +111,9 @@ public final class DefaultRulerModel implements RulerModel {
      * Line width
      */
     private double lineWidth = DEFAULT_LINE_WIDTH;
-    
+
+    private int subticksNumber = -1;
+
     /**
      * Default constructor.
      */
@@ -374,7 +377,7 @@ public final class DefaultRulerModel implements RulerModel {
     public double getLineWidth() {
         return lineWidth;
     }
-    
+
     /**
      * Ruler line width setter
      * @param lineWidth the new line width
@@ -383,4 +386,17 @@ public final class DefaultRulerModel implements RulerModel {
         this.lineWidth = lineWidth;
     }
 
+    @Override
+    public int getSubticksNumber() {
+        return subticksNumber;
+    }
+
+    /**
+     * Set number of subticks.
+     * @param N the number of subticks or -1 if the computation is automatic.
+     */
+    public void setSubticksNumber(int N) {
+        this.subticksNumber = N;
+    }
+
 }
