@@ -2,7 +2,8 @@
  * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
  * Copyright (C) 2009-2012 - DIGITEO - Pierre Lando
  * Copyright (C) 2012 - Scilab Enterprises - Bruno JOFRET
- *  *
+ * Copyright (C) 2013 - Scilab Enterprises - Calixte DENIZET
+ *
  * This file must be used under the terms of the CeCILL.
  * This source file is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -390,19 +391,24 @@ public class RulerDrawer {
          */
         private void computeTicksData() {
             if (graduations != null) {
-                density = graduations.getSubDensity();
                 ticksValue = graduations.getAllValues();
+                final int N = rulerModel.getSubticksNumber();
 
-                Graduations subGraduation = graduations.getSubGraduations();
+                if (N < 0) {
+                    Graduations subGraduation = graduations.getSubGraduations();
+                    while ((subGraduation != null) && (computeTicksDistance(subGraduation) < rulerModel.getMinimalSubTicksDistance())) {
+                        subGraduation = subGraduation.getAlternative();
+                    }
 
-                while ((subGraduation != null) && (computeTicksDistance(subGraduation) < rulerModel.getMinimalSubTicksDistance())) {
-                    subGraduation = subGraduation.getAlternative();
-                }
-                if (subGraduation != null) {
-                    subTicksValue = subGraduation.getAllValues();
+                    if (subGraduation != null) {
+                        subTicksValue = subGraduation.getAllValues();
+                    } else {
+                        subTicksValue = new LinkedList<Double>();
+                    }
                 } else {
-                    subTicksValue = new LinkedList<Double>();
+                    subTicksValue = graduations.getSubGraduations(N);
                 }
+                density = getDensity();
             } else {
                 subTicksValue = new LinkedList<Double>();
                 ticksValue = new LinkedList<Double>();
@@ -410,6 +416,17 @@ public class RulerDrawer {
             }
         }
 
+        private int getDensity() {
+            int N = ticksValue == null ? 0 : ticksValue.size();
+            int M = subTicksValue == null ? 0 : subTicksValue.size();
+
+            if (M <= N || N == 1) {
+                return 0;
+            }
+
+            return (M - N) / (N - 1);
+        }
+
         /**
          * Compute and return the minimal screen distance between two successive ticks of the given {@link Graduations}.
          * If the given {@link Graduations} is <code>null</code>, the returned value is {@link Double#MAX_VALUE}.
