@@ -11,8 +11,8 @@
 
 package org.scilab.forge.scirenderer.ruler;
 
-import org.scilab.forge.scirenderer.sprite.Sprite;
-import org.scilab.forge.scirenderer.sprite.SpriteManager;
+import org.scilab.forge.scirenderer.texture.Texture;
+import org.scilab.forge.scirenderer.texture.TextureManager;
 
 import java.text.DecimalFormat;
 
@@ -22,11 +22,11 @@ import java.text.DecimalFormat;
 public interface RulerSpriteFactory {
 
     /**
-     * Return the sprite for the given value.
+     * Return the texture for the given value.
      * @param value the value.
      * @param adaptedFormat an adapted number format for the given value.
-     * @param spriteManager {@link SpriteManager} to use.
+     * @param textureManager {@link TextureManager} to use.
      * @return a positioned text entity for the given value.
      */
-    Sprite create(double value, DecimalFormat adaptedFormat, SpriteManager spriteManager);
+    Texture create(double value, DecimalFormat adaptedFormat, TextureManager textureManager);
 }
