@@ -26,6 +26,9 @@ public class DefaultGeometry implements Geometry {
     private FillDrawingMode fillDrawingMode = DEFAULT_FILL_DRAWING_MODE;
     private LineDrawingMode lineDrawingMode = DEFAULT_LINE_DRAWING_MODE;
 
+    /** Specifies whether polygon offset is used or not when rendering geometry. */
+    private boolean polygonOffsetMode = DEFAULT_POLYGON_OFFSET_MODE;
+
     private ElementsBuffer textureCoordinates;
     private ElementsBuffer vertices;
     private ElementsBuffer normals;
@@ -55,6 +58,11 @@ public class DefaultGeometry implements Geometry {
         return lineDrawingMode;
     }
 
+    @Override
+    public final boolean getPolygonOffsetMode() {
+        return polygonOffsetMode;
+    }
+
     @Override
     public final ElementsBuffer getVertices() {
         return vertices;
@@ -109,6 +117,14 @@ public class DefaultGeometry implements Geometry {
         this.lineDrawingMode = lineDrawingMode;
     }
 
+    /**
+     * Polygon offset mode setter.
+     * @param polygonOffsetMode the new polygon offset mode.
+     */
+    public final void setPolygonOffsetMode(boolean polygonOffsetMode) {
+        this.polygonOffsetMode = polygonOffsetMode;
+    }
+
     /**
      * Texture coordinates setter.
      * @param textureCoordinates the new texture coordinate data.
