@@ -17,6 +17,7 @@ import java.awt.Dimension;
 import java.awt.Font;
 import java.awt.font.FontRenderContext;
 import java.awt.font.TextLayout;
+import java.awt.geom.Rectangle2D;
 
 /**
  * @author Pierre Lando
@@ -165,7 +166,7 @@ public class TextEntity {
         return ((getFont() != null)
                 && (getText() != null)
                 && (getText().length() > 0)
-        );
+               );
     }
 
     /**
@@ -177,8 +178,9 @@ public class TextEntity {
             FontRenderContext frc = new FontRenderContext(null, isTextAntiAliased(), isTextUseFractionalMetrics());
             TextLayout textLayout = new TextLayout(getText(), getFont(), frc);
             Dimension dimension = new Dimension();
+            Rectangle2D r = textLayout.getBounds();
             /* +1 added to fix rendering of ticks labels, a pixel row/column was missing */
-            dimension.setSize(textLayout.getBounds().getWidth() + 1, textLayout.getBounds().getHeight() + 1);
+            dimension.setSize(r.getWidth() + 2, r.getHeight() + 1);
             return dimension;
         } else {
             return new Dimension(0, 0);
