@@ -123,4 +123,18 @@ public interface Texture extends DataUser {
      * @param textureDrawer the given texture drawer.
      */
     void setDrawer(TextureDrawer textureDrawer);
+
+    /**
+     * 2D-Texture coordinates must be modified according to the real texture dimension which can differ from
+     * the textureSize (with certains GC, a texture must have a size which is a power-of-two).
+     * @return the scale factor for the s-coordinate
+     */
+    double getSScaleFactor();
+
+    /**
+     * 2D-Texture coordinates must be modified according to the real texture dimension which can differ from
+     * the textureSize (with certains GC, a texture must have a size which is a power-of-two).
+     * @return the scale factor for the t-coordinate
+     */
+    double getTScaleFactor();
 }
