@@ -14,6 +14,7 @@ package org.scilab.forge.scirenderer.texture;
 import org.scilab.forge.scirenderer.data.DataProvider;
 
 import java.awt.Dimension;
+import java.awt.image.BufferedImage;
 import java.nio.ByteBuffer;
 
 /**
@@ -42,4 +43,18 @@ public interface TextureDataProvider extends DataProvider<Texture> {
      * @return the data.
      */
     ByteBuffer getSubData(int x, int y, int width, int height);
+
+    /**
+     * @return the data as a BufferedImage
+     */
+    BufferedImage getImage();
+
+    /**
+     * @param x rectangle origin x-coordinate.
+     * @param y rectangle origin y-coordinate.
+     * @param width rectangle width.
+     * @param height rectangle height.
+     * @return the sub-data as a BufferedImage.
+     */
+    BufferedImage getSubImage(int x, int y, int width, int height);
 }
