@@ -11,6 +11,8 @@
 
 package org.scilab.forge.scirenderer.texture;
 
+import java.util.Collection;
+
 /**
  * @author Pierre Lando
  */
@@ -21,4 +23,12 @@ public interface TextureManager {
      * @return a new {@link Texture}
      */
     Texture createTexture();
-}
+
+    /**
+     * Dispose the given textures.
+     * @param textures textures to dispose.
+     */
+    void dispose(Collection<Texture> textures);
+
+    void dispose(Texture texture);
+}
\ No newline at end of file
