@@ -84,6 +84,37 @@ public class Rotation {
         this(alpha, v.asDouble());
     }
 
+    /**
+     * Get a rotation from an angle alpha given in degree and an axis v
+     * @param alpha angle of rotation in degree
+     * @param v the axis of the rotation
+     * @return a Rotation object
+     *
+     * For information, the function has been written to fix bug 11399.
+     * For angle (in radians) close to Pi the Rotation has a 'a' close to 0
+     * and that conflicts with small axes where length is closed to a: rounding errors
+     * led to a "bad" transformation matrix (AxesDrawer::computeBoxTransformation).
+     */
+    public static Rotation getDegreeRotation(double alpha, Vector3d v) {
+        int a = (int) alpha;
+        if (alpha == a) {
+            a = a % 360;
+            if (a == 0) {
+                return new Rotation(1, 0, 0, 0);
+            }
+            if (a == 180) {
+                Vector3d nv = v.getNormalized();
+                return new Rotation(0, nv.getX(), nv.getY(), nv.getZ());
+            }
+
+            return new Rotation(((double) a) / 180.0 * Math.PI, v);
+        }
+
+        alpha = (alpha / 180.0 - 2 * Math.floor(alpha / 360.0)) * Math.PI;
+
+        return new Rotation(alpha, v);
+    }
+
 
     private Rotation(double a, double b, double c, double d) {
         this.a = a;
@@ -144,10 +175,10 @@ public class Rotation {
     }
     public double[] getUnRotateMatrix() {
         return new double[] {
-                rotationMatrix[0], rotationMatrix[4], rotationMatrix[8], rotationMatrix[12],
-                rotationMatrix[1], rotationMatrix[5], rotationMatrix[9], rotationMatrix[13],
-                rotationMatrix[2], rotationMatrix[6], rotationMatrix[10], rotationMatrix[14],
-                rotationMatrix[3], rotationMatrix[7], rotationMatrix[11], rotationMatrix[15]
+            rotationMatrix[0], rotationMatrix[4], rotationMatrix[8], rotationMatrix[12],
+            rotationMatrix[1], rotationMatrix[5], rotationMatrix[9], rotationMatrix[13],
+            rotationMatrix[2], rotationMatrix[6], rotationMatrix[10], rotationMatrix[14],
+            rotationMatrix[3], rotationMatrix[7], rotationMatrix[11], rotationMatrix[15]
         };
     }
 
@@ -189,17 +220,17 @@ public class Rotation {
 
     public Vector3d conjugate(Vector3d v) {
         return new Vector3d(
-                rotationMatrix[0] * v.getX() + rotationMatrix[4] * v.getY() + rotationMatrix[8] * v.getZ(),
-                rotationMatrix[1] * v.getX() + rotationMatrix[5] * v.getY() + rotationMatrix[9] * v.getZ(),
-                rotationMatrix[2] * v.getX() + rotationMatrix[6] * v.getY() + rotationMatrix[10] * v.getZ()
-        );
+            rotationMatrix[0] * v.getX() + rotationMatrix[4] * v.getY() + rotationMatrix[8] * v.getZ(),
+            rotationMatrix[1] * v.getX() + rotationMatrix[5] * v.getY() + rotationMatrix[9] * v.getZ(),
+            rotationMatrix[2] * v.getX() + rotationMatrix[6] * v.getY() + rotationMatrix[10] * v.getZ()
+            );
     }
     public Vector3d conjugateInverse(Vector3d v) {
         return new Vector3d(
-                rotationMatrix[0] * v.getX() + rotationMatrix[1] * v.getY() + rotationMatrix[2] * v.getZ(),
-                rotationMatrix[4] * v.getX() + rotationMatrix[5] * v.getY() + rotationMatrix[6] * v.getZ(),
-                rotationMatrix[8] * v.getX() + rotationMatrix[9] * v.getY() + rotationMatrix[10] * v.getZ()
-        );
+            rotationMatrix[0] * v.getX() + rotationMatrix[1] * v.getY() + rotationMatrix[2] * v.getZ(),
+            rotationMatrix[4] * v.getX() + rotationMatrix[5] * v.getY() + rotationMatrix[6] * v.getZ(),
+            rotationMatrix[8] * v.getX() + rotationMatrix[9] * v.getY() + rotationMatrix[10] * v.getZ()
+            );
     }
 
     private void computeRotationMatrix() {
@@ -208,10 +239,10 @@ public class Rotation {
         double c2 = c * c;
         double d2 = d * d;
         this.rotationMatrix = new double[] {
-                a2 + b2 - c2 - d2,       2 * ((b * c) - (a * d)), 2 * ((a * c) + (b * d)), 0,
-                2 * ((a * d) + (b * c)), a2 - b2 + c2 - d2,       2 * ((c * d) - (a * b)), 0,
-                2 * ((b * d) - (a * c)), 2 * ((a * b) + (c * d)), a2 - b2 - c2 + d2,       0,
-                0,                       0,                       0,                       1
+            a2 + b2 - c2 - d2,       2 * ((b * c) - (a * d)), 2 * ((a * c) + (b * d)), 0,
+            2 * ((a * d) + (b * c)), a2 - b2 + c2 - d2,       2 * ((c * d) - (a * b)), 0,
+            2 * ((b * d) - (a * c)), 2 * ((a * b) + (c * d)), a2 - b2 - c2 + d2,       0,
+            0,                       0,                       0,                       1
         };
     }
 
