@@ -21,7 +21,7 @@ public final class TransformationFactory {
     /**
      * The identity transformation.
      */
-    private static final Transformation IDENTITY_TRANSFORMATION = new IdentityTransformation();
+    static final Transformation IDENTITY_TRANSFORMATION = new IdentityTransformation();
 
     /**
      * Default constructor.
@@ -111,7 +111,7 @@ public final class TransformationFactory {
         } else if (angle == 0) {
             return IDENTITY_TRANSFORMATION;
         } else {
-            return new RotationTransformation(new Rotation(Math.toRadians(angle), new Vector3d(x, y, z)));
+            return new RotationTransformation(Rotation.getDegreeRotation(angle, new Vector3d(x, y, z)));
         }
     }
 
@@ -284,10 +284,10 @@ public final class TransformationFactory {
          * The identity matrix data.
          */
         private static final double[] IDENTITY_MATRIX = new double[] {
-                1, 0, 0, 0,
-                0, 1, 0, 0,
-                0, 0, 1, 0,
-                0, 0, 0, 1
+            1, 0, 0, 0,
+            0, 1, 0, 0,
+            0, 0, 1, 0,
+            0, 0, 0, 1
         };
 
         /**
@@ -425,7 +425,7 @@ public final class TransformationFactory {
         private final double y;
 
         /**
-         * scale in z coordinate. 
+         * scale in z coordinate.
          */
         private final double z;
 
@@ -500,17 +500,17 @@ public final class TransformationFactory {
             double f    = 1 / fInv;
 
             matrix = new double[] {
-                    f, 0, 0,                             0,
-                    0, f, 0,                             0,
-                    0, 0, (far + near) / (near - far),   -1,
-                    0, 0, 2 * far * near / (near - far), 0
+                f, 0, 0,                             0,
+                0, f, 0,                             0,
+                0, 0, (far + near) / (near - far),   -1,
+                0, 0, 2 * far * near / (near - far), 0
             };
 
             inverseMatrix = new double[] {
-                    fInv, 0,    0,     0,
-                    0,    fInv, 0,     0,
-                    0,    0,    0,     (near - far) / (2 * far * near),
-                    0,    0,    -1,    (near + far) / (2 * far * near)
+                fInv, 0,    0,     0,
+                0,    fInv, 0,     0,
+                0,    0,    0,     (near - far) / (2 * far * near),
+                0,    0,    -1,    (near + far) / (2 * far * near)
             };
         }
 
@@ -554,17 +554,17 @@ public final class TransformationFactory {
             double tz = (far + near)   / (near - far);
 
             matrix = new double[] {
-                    2 / (right - left), 0,                  0,                  0,
-                    0,                  2 / (top - bottom), 0,                  0,
-                    0,                  0,                  2 / (near - far),   0,
-                    tx,                 ty,                 tz,                 1
+                2 / (right - left), 0,                  0,                  0,
+                0,                  2 / (top - bottom), 0,                  0,
+                0,                  0,                  2 / (near - far),   0,
+                tx,                 ty,                 tz,                 1
             };
 
             inverseMatrix = new double[] {
-                    (right - left) / 2, 0,                  0,                  0,
-                    0,                  (top - bottom) / 2, 0,                  0,
-                    0,                  0,                  (near - far) / 2,   0,
-                    (right + left) / 2, (top + bottom) / 2, -(near + far) / 2,  1
+                (right - left) / 2, 0,                  0,                  0,
+                0,                  (top - bottom) / 2, 0,                  0,
+                0,                  0,                  (near - far) / 2,   0,
+                (right + left) / 2, (top + bottom) / 2, -(near + far) / 2,  1
             };
 
             isIdentity = ((left == -1) && (right == 1) && (bottom == -1) && (top == 1) && (far == -1) && (near == 1));
