@@ -111,6 +111,21 @@ public class TransformationManagerImpl implements TransformationManager {
         }
     }
 
+    @Override
+    public Transformation getInverseWindowTransformation() {
+        double w = canvas.getWidth() / 2;
+        double h = canvas.getHeight() / 2;
+        try {
+            return TransformationFactory.getAffineTransformation(
+                new Vector3d(w, h, 1),
+                new Vector3d(w, h, 0)
+                );
+        } catch (DegenerateMatrixException e) {
+            // Should not occur as long as canvas have not infinite size.
+            throw new Error("Canvas is to big.", e);
+        }
+    }
+
     @Override
     public Transformation getCanvasProjection() {
         double w = canvas.getWidth() / 2.0;
